#!/bin/bash -e

# Tämä skripti demonstroi Bourne Again Shellin (bash)
# ohjelmointirakenteita siinä määrin kuin niitä esitellään
# Käyttöjärjestelmät -kurssilla. Oppimistavoite on kirjattu
# opinto-oppaaseen ylimalkaisesti "shell-ohjelmoinnin perusteita". No
# tässä nyt tulee joitakin perusteita.

# Taustalla on jo Demo1:ssä opitut tai ainakin nähdyt asiat eli
# yksittäisen shell-komennon antaminen, putkittaminen ja tulosteen
# ohjaus tiedostoon. Lisäksi Demo3:ssa nähtiin, kuinka komentosarja
# voidaan kirjoittaa suoritettavaksi tiedostoksi eli skriptiksi.

# Nyt lisätään ohjelmoinnin perusrakenteiden toteutuminen
# shellissä. Käytetään Bourne Again Shelliä; se on hyvä ja laajasti
# käytetty shell. Googlettamalla saa tarvittaessa lisää tietoa (kuten
# kaikesta)... Varsinaisia käyttötarkoituksia shell-skripteille on
# monia, mm. järjestelmän käynnistykseen ja sammuttamiseen liittyvät
# toimenpiteet, ajoitetut ylläpitotehtävät, automaattiset asennukset,
# käyttäjien login- ja log-off -asetukset ja toimenpiteet, nopeat
# prototyypit yksinkertaisista sovelluksista, toistuvien
# toimenpiteiden automatisointi.

# Yleishuomio: shell voi olla todella tarkka syntaksista,
# mm. välilyönnit ja rivinvaihdot ovat erityisen merkittäviä!  Ei
# mennä tässä syntaksiin tarkemmin kuin esimerkkikoodina, joka
# selvästi toimii ... Tentissä syntaksin yksityiskohdilla ei ole niin
# suuri merkitys, kuten ei ohjelmoinnissa yleensäkään ole kyse
# syntaksin vaan ohjelmoinnillisen rakenteen merkityksen
# ymmärtämisestä. Saatavilla on kuitenkin aina ohjekirjoja ja
# esimerkkejä, joista muistelemalla saa hommat toimimaan (edellyttäen
# että ymmärtää, mistä esimerkeissä on kyse -- ettei tule tehtyä
# tyhmyyksiä!!)

# Tässä jätetään vielä käsittelemättä paljon tärkeitä asioita,
# mm. aliohjelmat, kompaktimmat syntaksit, "hyvin käyttäytyvän"
# skriptin perusominaisuudet ja paljon muuta. Tutustu *halutessasi*
# esim. tutoriaaleihin, joihin demo-ohjeessa on linkit -- kurssin
# perussisällön mielessä ei tarvitse.

# Tulostaminen päätteelle
echo "------------- Esimerkkejä: ----------------------------"

# Argumentit
echo "Käynnistit skriptin komennolla: $0"
echo "Kaikki argumentit olivat: $@"
echo "Argumenttien lukumäärä oli: $#"
echo "Kaksi ensimmäistä argumenttia olivat: $1 $2"

# Hipsukoiden merkityksestä (katso tarkalla silmällä):
echo 'Mitäs "tästä" tulostuisi: $1 "$2"'
echo "Mitäs 'tästä' puolestaan: $1 '$2'"

# Ehtolause:
if [[ $1 == "" ]]
then
  echo "Et antanut yhtään argumenttia."
  echo "Pitää antaa ainakin yksi."
  exit 1
fi

# RegExp (HUOM: RegExpiä ^[0-9]+$ ei tarvitse ymmärtää jos ei halua!)
if ! [[ "$1" =~ ^[0-9]+$ ]]
then
  echo "Ensimmäisen argumentin tulee olla kokonaisluku!"
  exit 1
fi

# Vertailu (muuttujat on merkkijonoja; tulkitaan tarvittaessa
# numeroiksi; alla 'gt' tarkoittaa 'greater than' )
if [[ "$1" -gt "42" ]]
then
  echo "Annoit argumenttina aika ison luvun. Maksimi on 42."
  exit 1
fi


# Muuttujan asettaminen (välilyöntiä ei saa olla!):
outfname=tuloste.txt

maxcount=$1

outstr=""

# Silmukkarakenne, jossa lasketaan luvut ykkösestä ylöspäin
# argumenttina annettuun saakka ("-le" on "less than equal to"):
count=1
while [[ $count -le $maxcount ]]  # Muuttujan käytössä $ nimen eteen.
do
  outstr="$outstr $count"    # Merkkijonoon voitaisiin lisätä osia näin.
  let count=$count+1         # aritmetiikkaa on mahdollista tehdä näin.
done

echo "Muodostin juuri seuraavan merkkijonon:"
echo "$outstr"

echo "Käyn nyt läpi sen merkkijonon:"

# Silmukkarakenne, jossa käydään läpi välilyönneillä erotettu lista:
for alkio in $outstr
do
  echo $alkio
done


# Ehto, joka tarkistaa tiedoston tilanteen:
if [[ -f $outfname ]]
then
  echo "Tiedosto $outfname on jo olemassa!"
  # Sisäkkäiset rakenteet ovat tietysti mahdollisia...
  # Ehto, jossa on looginen AND eli optio -a:
  while true
  do
    echo -n "Korvataanko entinen (k/e)? "

    # Interaktiivinen skripti voisi lukea syötteen:
    read vastaus

    # Case-rakenne:
    case $vastaus in
	k|K)
	    echo "Tiedosto $outfname korvataan"
	    break
	    ;;
	e|E)
	    echo "Käyttäjä kieltäytyi jatkamasta.."
	    exit 1
	    ;;
	*)
	    echo "Vastaa k tai e"
	    ;;
    esac
  done
fi

# Jos suoritus päätyy tänne, on lupa korvata tulostustiedosto.
# Kirjoitetaan sinne siis:

echo "Ajettu ajanhetkellä `date`" > $outfname

# Ns. ympäristömuuttujat ovat aina asetettuja; niiden nimet on usein
# ISOILLA kirjaimilla. Kaikki shellistä ajetut ohjelmat saavat
# käyttöönsä vanhempiprosessin (eli shellin) ympäristön. Voit käyttää
# niitä Java- ja C-ohjelmista sekä skripteistä ja muistakin
# ohjelmista. Sama asia eli ympäristömuuttujat (environment variables)
# toteutuu myös muissa käyttöjärjestelmissä kuin Unixeissa.

echo "Ajavan käyttäjän kotihakemisto on $HOME" >> $outfname
echo "Tiedostojen etsintäpolku on $PATH" >> $outfname

echo "------------ ja nämä haluttiin tulostaa:" >> $outfname

# Tämä silmukka ja tiedostoonohjaus ovat samanlaisia kuin aiemmin:

for i in $outstr
do
  echo "Tämä on rivi nro $i ja se menee tiedostoon" >> $outfname
done

echo "Loppu. Tämän skriptin tuloste on tiedostossa $outfname"
